package com.main.controller;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.main.entity.Employee;
import com.main.producer.EmployeeProducer;
import com.main.service.EmployeeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/emp")
@Slf4j
public class EmployeeController {

    @Autowired
    private EmployeeService service;
    @Autowired
    private EmployeeProducer producer;



    @GetMapping("/getEmpById/{id}")
    public Mono<ResponseEntity<Employee>> getEmployeeById(@PathVariable Integer id){
        Mono<Employee> mono= service.getById(id)
                .doOnNext(emp-> {
                    try {
                        producer.sendEmployee(emp);
                    } catch (JsonProcessingException e) {
                        throw new RuntimeException(e);
                    }
                })
                .doOnNext(value -> log.info("Employee data get by id successfully: {}", value))
                .doOnError(error -> log.error("Data not found", error));



       return mono.map(ResponseEntity::ok)
               .defaultIfEmpty(ResponseEntity.notFound().build());
    }

    @GetMapping("/getAll")
    public Flux<Employee> getAllEmployee(){
        return service.getAllEmp();
    }
    @PostMapping("/saveEmp")
    public Mono<Employee> saveEmployee(@RequestBody Employee emp){
        Mono<Employee> mono= service.saveEmp(emp);

        return mono;
    }


    @GetMapping("/getByDeptId/{id}")
    public Flux<Employee> getByDeptId(@PathVariable Integer id) throws
            JsonProcessingException {
        // Create a Flux<Employee>
        Flux<Employee> flux = service.findByDeptId(id)
                .doOnNext(value -> log.info("Employee data fetched successfully: {}", value))
                .doOnError(error -> log.error("Data not found", error));

//         Collect employees into a list and send to producer
        flux.collectList()
                .doOnNext(list -> {
                    try {
                        producer.sendEmployee(list);
                    } catch (JsonProcessingException e) {
                        throw new RuntimeException(e);
                    }
                    log.info("Employee list sent to producer: {}", list);
                })
                .doOnError(error -> log.error("Error while sending employee list: {0}", error))
                .subscribe();

        return flux; // Return the Flux for further processing
    }



    }

