package com.main.service;


import com.main.entity.Employee;
import com.main.repository.EmployeeRepo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
@Slf4j
public class EmployeeService {

    @Autowired
    private EmployeeRepo repo;

    public Mono<Employee> getById(Integer id){
        Mono<Employee> mono=repo.findById(id)
                .doOnNext(value-> log.info("Value get by id  : {} ", value))
                .doOnError(error-> log.error("can't fetch data ", error));
        return mono;
    }
    public Flux<Employee> getAllEmp(){
        Flux<Employee> emps= repo.findAll()
                .doOnNext(value-> log.info("get all emp successfully : {}", value))
                .doOnError(error-> log.error("can't fetch all data ", error));
        return  emps;
    }
    public Mono<Employee> saveEmp(Employee emp){
        Mono<Employee> mono=repo.save(emp)
                .doOnNext(value-> log.info("save emp successfully : {}", value))
                .doOnError(error-> log.error("can't save all data ", error));
        return mono;
    }
    public Flux<Employee> findByDeptId(Integer id){
        Flux<Employee> emps= repo.findByDeptId(id)
                .doOnNext(value-> log.info("get emp by dept id successfully : {}", value))
                .doOnError(error-> log.error("can't fetch data by dept id ", error));
        return emps;
    }

}
